@@ -1,8 +1,7 @@
+import datetime
 import io
 from dataclasses import astuple
 from dataclasses import dataclass
-from datetime import datetime
-from datetime import timedelta
 from typing import List
 
 import pkg_resources
@@ -22,16 +21,6 @@ from flask_themer import render_template
 from werkzeug.datastructures import CombinedMultiDict
 from werkzeug.datastructures import FileStorage
 
-from .apputils import b64_to_obj
-from .apputils import default_fields
-from .apputils import login_placeholder
-from .apputils import obj_to_b64
-from .apputils import profile_hash
-from .flaskutils import current_user
-from .flaskutils import permissions_needed
-from .flaskutils import render_htmx_template
-from .flaskutils import smtp_needed
-from .flaskutils import user_needed
 from .forms import FirstLoginForm
 from .forms import ForgottenPasswordForm
 from .forms import InvitationForm
@@ -39,12 +28,23 @@ from .forms import LoginForm
 from .forms import PasswordForm
 from .forms import PasswordResetForm
 from .forms import profile_form
-from .forms import TableForm
 from .mails import send_invitation_mail
 from .mails import send_password_initialization_mail
 from .mails import send_password_reset_mail
 from .models import Group
 from .models import User
+from .utils import b64_to_obj
+from .utils import default_fields
+from .utils import login_placeholder
+from .utils import obj_to_b64
+from .utils import profile_hash
+from .utils.flask import current_user
+from .utils.flask import permissions_needed
+from .utils.flask import render_htmx_template
+from .utils.flask import request_is_htmx
+from .utils.flask import smtp_needed
+from .utils.flask import user_needed
+from .utils.forms import TableForm
 
 
 bp = Blueprint("account", __name__)
@@ -164,10 +164,7 @@ def firstlogin(uid):
     if not request.form:
         return render_template("firstlogin.html", form=form, uid=uid)
 
-    if not form.validate():
-        flash(_("Could not send the password initialization link."), "error")
-        return render_template("firstlogin.html", form=form, uid=uid)
-
+    form.validate()
     if send_password_initialization_mail(user):
         flash(
             _(
@@ -205,11 +202,13 @@ class Invitation:
 
     @property
     def creation_date(self):
-        return datetime.fromisoformat(self.creation_date_isoformat)
+        return datetime.datetime.fromisoformat(self.creation_date_isoformat)
 
     def has_expired(self):
         DEFAULT_INVITATION_DURATION = 2 * 24 * 60 * 60
-        return datetime.now() - self.creation_date > timedelta(
+        return datetime.datetime.now(
+            datetime.timezone.utc
+        ) - self.creation_date > datetime.timedelta(
             seconds=current_app.config.get(
                 "INVITATION_EXPIRATION", DEFAULT_INVITATION_DURATION
             )
@@ -234,7 +233,7 @@ def user_invitation(user):
     if request.form and form.validate():
         form_validated = True
         invitation = Invitation(
-            datetime.now().isoformat(),
+            datetime.datetime.now(datetime.timezone.utc).isoformat(),
             form.uid.data,
             form.uid_editable.data,
             form.mail.data,
@@ -260,33 +259,6 @@ def user_invitation(user):
     )
 
 
-@bp.route("/profile", methods=("GET", "POST"))
-@permissions_needed("manage_users")
-def profile_creation(user):
-    form = profile_form(user.write, user.read)
-    form.process(CombinedMultiDict((request.files, request.form)) or None)
-
-    for field in form:
-        if field.render_kw and "readonly" in field.render_kw:
-            del field.render_kw["readonly"]
-
-    if request.form:
-        if not form.validate():
-            flash(_("User account creation failed."), "error")
-
-        else:
-            user = profile_create(current_app, form)
-            return redirect(url_for("account.profile_edition", username=user.uid[0]))
-
-    return render_template(
-        "profile.html",
-        form=form,
-        menuitem="users",
-        edited_user=None,
-        self_deletion=False,
-    )
-
-
 @bp.route("/register/<data>/<hash>", methods=["GET", "POST"])
 def registration(data, hash):
     try:
@@ -360,11 +332,38 @@ def registration(data, hash):
         else:
             user = profile_create(current_app, form)
             user.login()
-            flash(_("You account has been created successfuly."), "success")
+            flash(_("Your account has been created successfuly."), "success")
             return redirect(url_for("account.profile_edition", username=user.uid[0]))
 
     return render_template(
-        "profile.html",
+        "profile_add.html",
+        form=form,
+        menuitem="users",
+        edited_user=None,
+        self_deletion=False,
+    )
+
+
+@bp.route("/profile", methods=("GET", "POST"))
+@permissions_needed("manage_users")
+def profile_creation(user):
+    form = profile_form(user.write, user.read)
+    form.process(CombinedMultiDict((request.files, request.form)) or None)
+
+    for field in form:
+        if field.render_kw and "readonly" in field.render_kw:
+            del field.render_kw["readonly"]
+
+    if request.form:
+        if not form.validate():
+            flash(_("User account creation failed."), "error")
+
+        else:
+            user = profile_create(current_app, form)
+            return redirect(url_for("account.profile_edition", username=user.uid[0]))
+
+    return render_template(
+        "profile_add.html",
         form=form,
         menuitem="users",
         edited_user=None,
@@ -407,55 +406,12 @@ def profile_create(current_app, form):
 @bp.route("/profile/<username>", methods=("GET", "POST"))
 @user_needed()
 def profile_edition(user, username):
+    editor = user
     if not user.can_manage_users and not (
         user.can_edit_self and username == user.uid[0]
     ):
         abort(403)
 
-    if request.method == "GET" or request.form.get("action") == "edit":
-        return profile_edit(user, username)
-
-    if request.form.get("action") == "delete":
-        return profile_delete(user, username)
-
-    if request.form.get("action") == "password-initialization-mail":
-        user = User.get(username)
-        if not user:
-            abort(404)
-
-        if send_password_initialization_mail(user):
-            flash(
-                _(
-                    "A password initialization link has been sent at the user email address. It should be received within a few minutes."
-                ),
-                "success",
-            )
-        else:
-            flash(_("Could not send the password initialization email"), "error")
-
-        return profile_edit(user, username)
-
-    if request.form.get("action") == "password-reset-mail":
-        user = User.get(username)
-        if not user:
-            abort(404)
-
-        if send_password_reset_mail(user):
-            flash(
-                _(
-                    "A password reset link has been sent at the user email address. It should be received within a few minutes."
-                ),
-                "success",
-            )
-        else:
-            flash(_("Could not send the password reset email"), "error")
-
-        return profile_edit(user, username)
-
-    abort(400)
-
-
-def profile_edit(editor, username):
     menuitem = "profile" if username == editor.uid[0] else "users"
     fields = editor.read | editor.write
     if username != editor.uid[0]:
@@ -466,18 +422,38 @@ def profile_edit(editor, username):
     if not user:
         abort(404)
 
+    available_fields = {
+        "cn",
+        "title",
+        "givenName",
+        "sn",
+        "displayName",
+        "mail",
+        "telephoneNumber",
+        "postalAddress",
+        "street",
+        "postalCode",
+        "l",
+        "st",
+        "jpegPhoto",
+        "jpegPhoto_delete",
+        "employeeNumber",
+        "departmentNumber",
+        "labeledURI",
+        "preferredLanguage",
+        "o",
+    }
     data = {
         k: getattr(user, k)[0]
         if getattr(user, k) and isinstance(getattr(user, k), list)
         else getattr(user, k) or ""
         for k in fields
-        if hasattr(user, k)
+        if hasattr(user, k) and k in available_fields
     }
 
-    if "groups" in fields:
-        data["groups"] = [g.id for g in user.groups]
-
-    form = profile_form(editor.write, editor.read)
+    form = profile_form(
+        editor.write & available_fields, editor.read & available_fields, user
+    )
     form.process(CombinedMultiDict((request.files, request.form)) or None, data=data)
 
     if request.form:
@@ -497,19 +473,9 @@ def profile_edit(editor, username):
 
                     user[attribute.name] = data
 
-                elif attribute.name == "groups" and "groups" in editor.write:
-                    user.set_groups(attribute.data)
-
             if "jpegPhoto" in form and form["jpegPhoto_delete"].data:
                 user["jpegPhoto"] = None
 
-            if (
-                "password1" in request.form
-                and form["password1"].data
-                and request.form["action"] == "edit"
-            ):
-                user.set_password(form["password1"].data)
-
             if "preferredLanguage" in request.form:
                 # Refresh the babel cache in case the lang is updated
                 refresh()
@@ -522,26 +488,126 @@ def profile_edit(editor, username):
             return redirect(url_for("account.profile_edition", username=username))
 
     return render_template(
-        "profile.html",
+        "profile_edit.html",
         form=form,
         menuitem=menuitem,
         edited_user=user,
-        self_deletion=user.can_delete_account,
     )
 
 
-def profile_delete(user, username):
-    self_deletion = username == user.uid[0]
-    if self_deletion:
-        user.logout()
-    else:
-        user = User.get(username)
+@bp.route("/profile/<username>/settings", methods=("GET", "POST"))
+@user_needed()
+def profile_settings(user, username):
+    if not user.can_manage_users and not (
+        user.can_edit_self and username == user.uid[0]
+    ):
+        abort(403)
 
-    if not user:
+    edited_user = User.get(username)
+    if not edited_user:
         abort(404)
 
-    flash(_("The user %(user)s has been sucessfuly deleted", user=user.name), "success")
-    user.delete()
+    if (
+        request.method == "GET"
+        or request.form.get("action") == "edit"
+        or request_is_htmx()
+    ):
+        return profile_settings_edit(user, edited_user)
+
+    if request.form.get("action") == "delete":
+        return profile_delete(user, edited_user)
+
+    if request.form.get("action") == "password-initialization-mail":
+        if send_password_initialization_mail(user):
+            flash(
+                _(
+                    "A password initialization link has been sent at the user email address. It should be received within a few minutes."
+                ),
+                "success",
+            )
+        else:
+            flash(_("Could not send the password initialization email"), "error")
+
+        return profile_settings_edit(user, edited_user)
+
+    if request.form.get("action") == "password-reset-mail":
+        if send_password_reset_mail(user):
+            flash(
+                _(
+                    "A password reset link has been sent at the user email address. It should be received within a few minutes."
+                ),
+                "success",
+            )
+        else:
+            flash(_("Could not send the password reset email"), "error")
+
+        return profile_settings_edit(user, edited_user)
+
+    abort(400)
+
+
+def profile_settings_edit(editor, edited_user):
+    menuitem = "profile" if editor.id == editor.id else "users"
+    fields = editor.read | editor.write
+
+    available_fields = {"userPassword", "groups", "uid"}
+    data = {
+        k: getattr(edited_user, k)[0]
+        if getattr(edited_user, k) and isinstance(getattr(edited_user, k), list)
+        else getattr(edited_user, k) or ""
+        for k in fields
+        if hasattr(edited_user, k) and k in available_fields
+    }
+
+    if "groups" in fields:
+        data["groups"] = [g.id for g in edited_user.groups]
+
+    form = profile_form(
+        editor.write & available_fields, editor.read & available_fields, edited_user
+    )
+    form.process(CombinedMultiDict((request.files, request.form)) or None, data=data)
+
+    if request.form and request.form.get("action") == "edit" or request_is_htmx():
+        if not form.validate():
+            flash(_("Profile edition failed."), "error")
+
+        else:
+            for attribute in form:
+                if attribute.name == "groups" and "groups" in editor.write:
+                    edited_user.set_groups(attribute.data)
+
+            if (
+                "password1" in request.form
+                and form["password1"].data
+                and request.form["action"] == "edit"
+            ):
+                edited_user.set_password(form["password1"].data)
+
+            edited_user.save()
+            flash(_("Profile updated successfuly."), "success")
+            return redirect(
+                url_for("account.profile_edition", username=edited_user.uid[0])
+            )
+
+    return render_template(
+        "profile_settings.html",
+        form=form,
+        menuitem=menuitem,
+        edited_user=edited_user,
+        self_deletion=edited_user.can_delete_account,
+    )
+
+
+def profile_delete(user, edited_user):
+    self_deletion = user.id == edited_user.id
+    if self_deletion:
+        user.logout()
+
+    flash(
+        _("The user %(user)s has been sucessfuly deleted", user=edited_user.name),
+        "success",
+    )
+    edited_user.delete()
 
     if self_deletion:
         return redirect(url_for("account.index"))
@@ -556,6 +622,8 @@ def impersonate(user, username):
         abort(404)
 
     puppet.login()
+
+    flash(_("Connection successful. Welcome %(user)s", user=puppet.name), "success")
     return redirect(url_for("account.index"))
 
 
@@ -643,9 +711,19 @@ def photo(uid, field):
     if not user:
         abort(404)
 
+    etag = None
+    if request.if_modified_since and request.if_modified_since >= user.modifyTimestamp:
+        return "", 304
+
+    etag = profile_hash(uid, user.modifyTimestamp.isoformat())
+    if request.if_none_match and etag in request.if_none_match:
+        return "", 304
+
     photos = getattr(user, field)
     if not photos:
         abort(404)
 
     stream = io.BytesIO(photos[0])
-    return send_file(stream, mimetype="image/jpeg")
+    return send_file(
+        stream, mimetype="image/jpeg", last_modified=user.modifyTimestamp, etag=etag
+    )
