@@ -1,29 +1,37 @@
-import math
-
 import wtforms.form
+from canaille.utils.flask import request_is_htmx
+from flask import abort
 from flask import current_app
 from flask import g
+from flask import make_response
+from flask import request
 from flask_babel import lazy_gettext as _
-from flask_wtf import FlaskForm
 from flask_wtf.file import FileAllowed
 from flask_wtf.file import FileField
 
 from .i18n import native_language_name_from_code
 from .models import Group
 from .models import User
+from .utils.forms import HTMXBaseForm
+from .utils.forms import HTMXForm
+from .utils.forms import is_uri
 
 
 def unique_login(form, field):
-    if User.get(field.data):
+    if User.get(field.data) and (
+        not getattr(form, "user", None) or form.user.uid[0] != field.data
+    ):
         raise wtforms.ValidationError(
             _("The login '{login}' already exists").format(login=field.data)
         )
 
 
 def unique_email(form, field):
-    if User.get(filter=f"(mail={field.data})"):
+    if User.get(mail=field.data) and (
+        not getattr(form, "user", None) or form.user.mail[0] != field.data
+    ):
         raise wtforms.ValidationError(
-            _("The email '{email}' already exists").format(email=field.data)
+            _("The email '{email}' is already used").format(email=field.data)
         )
 
 
@@ -43,31 +51,7 @@ def existing_login(form, field):
         )
 
 
-class TableForm(FlaskForm):
-    def __init__(self, cls=None, page_size=25, fields=None, filter=None, **kwargs):
-        filter = filter or {}
-        super().__init__(**kwargs)
-        if self.query.data:
-            self.items = cls.fuzzy(self.query.data, fields, **filter)
-        else:
-            self.items = cls.query(**filter)
-
-        self.page_size = page_size
-        self.nb_items = len(self.items)
-        self.page_max = max(1, math.ceil(self.nb_items / self.page_size))
-        first_item = (self.page.data - 1) * self.page_size
-        last_item = min((self.page.data) * self.page_size, self.nb_items)
-        self.items_slice = self.items[first_item:last_item]
-
-    page = wtforms.IntegerField(default=1)
-    query = wtforms.StringField(default="")
-
-    def validate_page(self, field):
-        if field.data < 1 or field.data > self.page_max:
-            raise wtforms.validators.ValidationError(_("The page number is not valid"))
-
-
-class LoginForm(FlaskForm):
+class LoginForm(HTMXForm):
     login = wtforms.StringField(
         _("Login"),
         validators=[wtforms.validators.DataRequired(), existing_login],
@@ -80,7 +64,7 @@ class LoginForm(FlaskForm):
     )
 
 
-class PasswordForm(FlaskForm):
+class PasswordForm(HTMXForm):
     password = wtforms.PasswordField(
         _("Password"),
         validators=[wtforms.validators.DataRequired()],
@@ -91,7 +75,7 @@ class FullLoginForm(LoginForm, PasswordForm):
     pass
 
 
-class ForgottenPasswordForm(FlaskForm):
+class ForgottenPasswordForm(HTMXForm):
     login = wtforms.StringField(
         _("Login"),
         validators=[wtforms.validators.DataRequired(), existing_login],
@@ -103,9 +87,13 @@ class ForgottenPasswordForm(FlaskForm):
     )
 
 
-class PasswordResetForm(FlaskForm):
+class PasswordResetForm(HTMXForm):
     password = wtforms.PasswordField(
-        _("Password"), validators=[wtforms.validators.DataRequired()]
+        _("Password"),
+        validators=[wtforms.validators.DataRequired()],
+        render_kw={
+            "autocomplete": "new-password",
+        },
     )
     confirmation = wtforms.PasswordField(
         _("Password confirmation"),
@@ -114,10 +102,13 @@ class PasswordResetForm(FlaskForm):
                 "password", _("Password and confirmation do not match.")
             ),
         ],
+        render_kw={
+            "autocomplete": "new-password",
+        },
     )
 
 
-class FirstLoginForm(FlaskForm):
+class FirstLoginForm(HTMXForm):
     pass
 
 
@@ -134,10 +125,12 @@ PROFILE_FORM_FIELDS = dict(
     uid=wtforms.StringField(
         _("Username"),
         render_kw={"placeholder": _("jdoe")},
-        validators=[wtforms.validators.DataRequired()],
+        validators=[wtforms.validators.DataRequired(), unique_login],
     ),
     cn=wtforms.StringField(_("Name")),
-    title=wtforms.TelField(_("Title"), render_kw={"placeholder": _("Vice president")}),
+    title=wtforms.StringField(
+        _("Title"), render_kw={"placeholder": _("Vice president")}
+    ),
     givenName=wtforms.StringField(
         _("Given name"),
         render_kw={
@@ -166,7 +159,11 @@ PROFILE_FORM_FIELDS = dict(
     ),
     mail=wtforms.EmailField(
         _("Email address"),
-        validators=[wtforms.validators.DataRequired(), wtforms.validators.Email()],
+        validators=[
+            wtforms.validators.DataRequired(),
+            wtforms.validators.Email(),
+            unique_email,
+        ],
         description=_(
             "This email will be used as a recovery address to reset the password if needed"
         ),
@@ -218,6 +215,9 @@ PROFILE_FORM_FIELDS = dict(
     password1=wtforms.PasswordField(
         _("Password"),
         validators=[wtforms.validators.Optional(), wtforms.validators.Length(min=8)],
+        render_kw={
+            "autocomplete": "new-password",
+        },
     ),
     password2=wtforms.PasswordField(
         _("Password confirmation"),
@@ -226,9 +226,12 @@ PROFILE_FORM_FIELDS = dict(
                 "password1", message=_("Password and confirmation do not match.")
             )
         ],
+        render_kw={
+            "autocomplete": "new-password",
+        },
     ),
     employeeNumber=wtforms.StringField(
-        _("Employee number"),
+        _("User number"),
         render_kw={
             "placeholder": _("1234"),
         },
@@ -239,20 +242,32 @@ PROFILE_FORM_FIELDS = dict(
             "placeholder": _("1234"),
         },
     ),
+    o=wtforms.StringField(
+        _("Organization"),
+        render_kw={
+            "placeholder": _("Cogip LTD."),
+        },
+    ),
     labeledURI=wtforms.URLField(
         _("Website"),
         render_kw={
             "placeholder": _("https://mywebsite.tld"),
         },
+        validators=[wtforms.validators.Optional(), is_uri],
     ),
     preferredLanguage=wtforms.SelectField(
         _("Preferred language"),
         choices=available_language_choices,
     ),
+    groups=wtforms.SelectMultipleField(
+        _("Groups"),
+        choices=lambda: [(group.id, group.display_name) for group in Group.query()],
+        render_kw={"placeholder": _("users, admins …")},
+    ),
 )
 
 
-def profile_form(write_field_names, readonly_field_names):
+def profile_form(write_field_names, readonly_field_names, user=None):
     if "userPassword" in write_field_names:
         write_field_names |= {"password1", "password2"}
 
@@ -265,14 +280,11 @@ def profile_form(write_field_names, readonly_field_names):
         if PROFILE_FORM_FIELDS.get(name)
     }
 
-    if "groups" in write_field_names | readonly_field_names and Group.query():
-        fields["groups"] = wtforms.SelectMultipleField(
-            _("Groups"),
-            choices=[(group.id, group.display_name) for group in Group.query()],
-            render_kw={"placeholder": _("users, admins …")},
-        )
+    if "groups" in fields and not Group.query():
+        del fields["groups"]
 
-    form = wtforms.form.BaseForm(fields)
+    form = HTMXBaseForm(fields)
+    form.user = user
     for field in form:
         if field.name in readonly_field_names - write_field_names:
             field.render_kw["readonly"] = "true"
@@ -280,7 +292,7 @@ def profile_form(write_field_names, readonly_field_names):
     return form
 
 
-class CreateGroupForm(FlaskForm):
+class CreateGroupForm(HTMXForm):
     display_name = wtforms.StringField(
         _("Name"),
         validators=[wtforms.validators.DataRequired(), unique_group],
@@ -294,7 +306,7 @@ class CreateGroupForm(FlaskForm):
     )
 
 
-class EditGroupForm(FlaskForm):
+class EditGroupForm(HTMXForm):
     display_name = wtforms.StringField(
         _("Name"),
         validators=[wtforms.validators.DataRequired()],
@@ -308,7 +320,7 @@ class EditGroupForm(FlaskForm):
     )
 
 
-class InvitationForm(FlaskForm):
+class InvitationForm(HTMXForm):
     uid = wtforms.StringField(
         _("Username"),
         render_kw={"placeholder": _("jdoe")},
