@@ -7,13 +7,13 @@ from flask import url_for
 from flask_babel import gettext as _
 from flask_themer import render_template
 
-from .flaskutils import permissions_needed
-from .flaskutils import render_htmx_template
 from .forms import CreateGroupForm
 from .forms import EditGroupForm
-from .forms import TableForm
 from .models import Group
 from .models import User
+from .utils.flask import permissions_needed
+from .utils.flask import render_htmx_template
+from .utils.forms import TableForm
 
 bp = Blueprint("groups", __name__, url_prefix="/groups")
 
@@ -51,7 +51,9 @@ def create_group(user):
             )
             return redirect(url_for("groups.group", groupname=group.display_name))
 
-    return render_template("group.html", form=form, edited_group=None, members=None)
+    return render_template(
+        "group.html", menuitem="groups", form=form, edited_group=None, members=None
+    )
 
 
 @bp.route("/<groupname>", methods=("GET", "POST"))
@@ -107,6 +109,7 @@ def edit_group(group):
         "group.html",
         "partial/users.html",
         form=form,
+        menuitem="groups",
         edited_group=group,
         table_form=table_form,
     )
