@@ -27,7 +27,7 @@ def setup_i18n(app):
 
 
 def locale_selector():
-    from .flaskutils import current_user
+    from .utils.flask import current_user
 
     user = current_user()
     available_language_codes = getattr(g, "available_language_codes", [])
