@@ -18,8 +18,13 @@ def setup_ldap_models(config):
 
     user_base = config["LDAP"]["USER_BASE"].replace(f',{config["LDAP"]["ROOT_DN"]}', "")
     User.base = user_base
-    User.id = config["LDAP"].get("USER_ID_ATTRIBUTE", User.DEFAULT_ID_ATTRIBUTE)
-    User.object_class = [config["LDAP"].get("USER_CLASS", User.DEFAULT_OBJECT_CLASS)]
+    User.rdn_attribute = config["LDAP"].get(
+        "USER_ID_ATTRIBUTE", User.DEFAULT_ID_ATTRIBUTE
+    )
+    object_class = config["LDAP"].get("USER_CLASS", User.DEFAULT_OBJECT_CLASS)
+    User.ldap_object_class = (
+        object_class if isinstance(object_class, list) else [object_class]
+    )
 
     group_base = (
         config["LDAP"]
@@ -27,8 +32,13 @@ def setup_ldap_models(config):
         .replace(f',{config["LDAP"]["ROOT_DN"]}', "")
     )
     Group.base = group_base or None
-    Group.id = config["LDAP"].get("GROUP_ID_ATTRIBUTE", Group.DEFAULT_ID_ATTRIBUTE)
-    Group.object_class = [config["LDAP"].get("GROUP_CLASS", Group.DEFAULT_OBJECT_CLASS)]
+    Group.rdn_attribute = config["LDAP"].get(
+        "GROUP_ID_ATTRIBUTE", Group.DEFAULT_ID_ATTRIBUTE
+    )
+    object_class = config["LDAP"].get("GROUP_CLASS", Group.DEFAULT_OBJECT_CLASS)
+    Group.ldap_object_class = (
+        object_class if isinstance(object_class, list) else [object_class]
+    )
 
 
 def setup_backend(app):
@@ -39,10 +49,11 @@ def setup_backend(app):
         pass
 
     try:
-        g.ldap = ldap.initialize(app.config["LDAP"]["URI"])
-        if app.config["LDAP"].get("TIMEOUT"):
-            g.ldap.set_option(ldap.OPT_NETWORK_TIMEOUT, app.config["LDAP"]["TIMEOUT"])
-        g.ldap.simple_bind_s(
+        g.ldap_connection = ldap.initialize(app.config["LDAP"]["URI"])
+        g.ldap_connection.set_option(
+            ldap.OPT_NETWORK_TIMEOUT, app.config["LDAP"].get("TIMEOUT")
+        )
+        g.ldap_connection.simple_bind_s(
             app.config["LDAP"]["BIND_DN"], app.config["LDAP"]["BIND_PW"]
         )
 
@@ -80,9 +91,9 @@ def setup_backend(app):
 
 
 def teardown_backend(app):
-    if g.get("ldap"):
-        g.ldap.unbind_s()
-        g.ldap = None
+    if g.get("ldap_connection"):  # pragma: no branch
+        g.ldap_connection.unbind_s()
+        g.ldap_connection = None
 
 
 def init_backend(app):
@@ -105,8 +116,7 @@ def validate_configuration(config):
 
     try:
         conn = ldap.initialize(config["LDAP"]["URI"])
-        if config["LDAP"].get("TIMEOUT"):
-            conn.set_option(ldap.OPT_NETWORK_TIMEOUT, config["LDAP"]["TIMEOUT"])
+        conn.set_option(ldap.OPT_NETWORK_TIMEOUT, config["LDAP"].get("TIMEOUT"))
         conn.simple_bind_s(config["LDAP"]["BIND_DN"], config["LDAP"]["BIND_PW"])
 
     except ldap.SERVER_DOWN as exc:
@@ -122,7 +132,6 @@ def validate_configuration(config):
     try:
         User.ldap_object_classes(conn)
         user = User(
-            objectClass=["inetOrgPerson"],
             cn=f"canaille_{uuid.uuid4()}",
             sn=f"canaille_{uuid.uuid4()}",
             uid=f"canaille_{uuid.uuid4()}",
@@ -142,7 +151,6 @@ def validate_configuration(config):
         Group.ldap_object_classes(conn)
 
         user = User(
-            objectClass=["inetOrgPerson"],
             cn=f"canaille_{uuid.uuid4()}",
             sn=f"canaille_{uuid.uuid4()}",
             uid=f"canaille_{uuid.uuid4()}",
@@ -152,9 +160,8 @@ def validate_configuration(config):
         user.save(conn)
 
         group = Group(
-            objectClass=["groupOfNames"],
-            cn=f"canaille_{uuid.uuid4()}",
-            member=[user.dn],
+            display_name=f"canaille_{uuid.uuid4()}",
+            members=[user],
         )
         group.save(conn)
         group.delete(conn)
