@@ -1,7 +1,7 @@
-from canaille.flaskutils import permissions_needed
-from canaille.flaskutils import render_htmx_template
-from canaille.forms import TableForm
 from canaille.oidc.models import AuthorizationCode
+from canaille.utils.flask import permissions_needed
+from canaille.utils.flask import render_htmx_template
+from canaille.utils.forms import TableForm
 from flask import abort
 from flask import Blueprint
 from flask import request
