@@ -1,10 +1,11 @@
 import datetime
 
-from canaille.flaskutils import permissions_needed
-from canaille.flaskutils import render_htmx_template
-from canaille.forms import TableForm
-from canaille.oidc.forms import ClientAdd
+from canaille.oidc.forms import ClientAddForm
 from canaille.oidc.models import Client
+from canaille.utils.flask import permissions_needed
+from canaille.utils.flask import render_htmx_template
+from canaille.utils.flask import request_is_htmx
+from canaille.utils.forms import TableForm
 from flask import abort
 from flask import Blueprint
 from flask import flash
@@ -34,7 +35,7 @@ def index(user):
 @bp.route("/add", methods=["GET", "POST"])
 @permissions_needed("manage_oidc")
 def add(user):
-    form = ClientAdd(request.form or None)
+    form = ClientAddForm(request.form or None)
 
     if not request.form:
         return render_template(
@@ -51,7 +52,7 @@ def add(user):
         )
 
     client_id = gen_salt(24)
-    client_id_issued_at = datetime.datetime.now()
+    client_id_issued_at = datetime.datetime.now(datetime.timezone.utc)
     client = Client(
         client_id=client_id,
         client_id_issued_at=client_id_issued_at,
@@ -89,7 +90,11 @@ def add(user):
 @bp.route("/edit/<client_id>", methods=["GET", "POST"])
 @permissions_needed("manage_oidc")
 def edit(user, client_id):
-    if request.method == "GET" or request.form.get("action") == "edit":
+    if (
+        request.method == "GET"
+        or request.form.get("action") == "edit"
+        or request_is_htmx()
+    ):
         return client_edit(client_id)
 
     if request.form.get("action") == "delete":
@@ -114,7 +119,7 @@ def client_edit(client_id):
         else ""
     )
     data["preconsent"] = client.preconsent
-    form = ClientAdd(request.form or None, data=data, client=client)
+    form = ClientAddForm(request.form or None, data=data, client=client)
 
     if not request.form:
         return render_template(
