@@ -5,6 +5,7 @@ from authlib.integrations.flask_oauth2 import current_token
 from authlib.jose import JsonWebKey
 from authlib.jose import jwt
 from authlib.oauth2 import OAuth2Error
+from canaille import csrf
 from flask import abort
 from flask import Blueprint
 from flask import current_app
@@ -18,10 +19,11 @@ from flask_babel import gettext as _
 from flask_themer import render_template
 from werkzeug.datastructures import CombinedMultiDict
 
-from ..flaskutils import current_user
-from ..flaskutils import set_parameter_in_url_query
 from ..forms import FullLoginForm
 from ..models import User
+from ..utils.flask import current_user
+from ..utils.flask import set_parameter_in_url_query
+from .forms import AuthorizeForm
 from .forms import LogoutForm
 from .models import Client
 from .models import Consent
@@ -115,6 +117,7 @@ def authorize():
             current_app.logger.debug("authorization endpoint response: %s", response)
             return jsonify(response)
 
+        form = AuthorizeForm(request.form or None)
         return render_template(
             "oidc/user/authorize.html",
             user=user,
@@ -123,6 +126,7 @@ def authorize():
             menu=False,
             scope_details=SCOPE_DETAILS,
             ignored_scopes=["openid"],
+            form=form,
         )
 
     # request.method == "POST"
@@ -149,7 +153,7 @@ def authorize():
                 client=client,
                 subject=user,
                 scope=scopes,
-                issue_date=datetime.datetime.now(),
+                issue_date=datetime.datetime.now(datetime.timezone.utc),
             )
         consent.save()
 
@@ -159,6 +163,7 @@ def authorize():
 
 
 @bp.route("/token", methods=["POST"])
+@csrf.exempt
 def issue_token():
     current_app.logger.debug(
         "token endpoint request: POST: %s", request.form.to_dict(flat=False)
@@ -169,6 +174,7 @@ def issue_token():
 
 
 @bp.route("/introspect", methods=["POST"])
+@csrf.exempt
 def introspect_token():
     current_app.logger.debug(
         "introspection endpoint request: POST: %s", request.form.to_dict(flat=False)
@@ -181,6 +187,7 @@ def introspect_token():
 
 
 @bp.route("/revoke", methods=["POST"])
+@csrf.exempt
 def revoke_token():
     current_app.logger.debug(
         "revokation endpoint request: POST: %s", request.form.to_dict(flat=False)
@@ -191,6 +198,7 @@ def revoke_token():
 
 
 @bp.route("/register", methods=["POST"])
+@csrf.exempt
 def client_registration():
     current_app.logger.debug(
         "client registration endpoint request: POST: %s",
@@ -204,6 +212,7 @@ def client_registration():
 
 
 @bp.route("/register/<client_id>", methods=["GET", "PUT", "DELETE"])
+@csrf.exempt
 def client_registration_management(client_id):
     current_app.logger.debug(
         "client registration management endpoint request: POST: %s",
@@ -336,10 +345,7 @@ def end_session():
 @bp.route("/end_session_confirm", methods=["POST"])
 def end_session_submit():
     form = LogoutForm(request.form)
-    if not form.validate():
-        flash(_("An error happened during the logout"), "error")
-        client = Client.get(session.get("end_session_data", {}).get("client_id"))
-        return render_template("oidc/user/logout.html", form=form, client=client)
+    form.validate()
 
     data = session["end_session_data"]
     del session["end_session_data"]
