@@ -1,10 +1,15 @@
 import wtforms
 from canaille.oidc.models import Client
+from canaille.utils.forms import HTMXForm
+from canaille.utils.forms import is_uri
 from flask_babel import lazy_gettext as _
-from flask_wtf import FlaskForm
 
 
-class LogoutForm(FlaskForm):
+class AuthorizeForm(HTMXForm):
+    pass
+
+
+class LogoutForm(HTMXForm):
     answer = wtforms.SubmitField()
 
 
@@ -12,7 +17,7 @@ def client_audiences():
     return [(client.id, client.client_name) for client in Client.query()]
 
 
-class ClientAdd(FlaskForm):
+class ClientAddForm(HTMXForm):
     client_name = wtforms.StringField(
         _("Name"),
         validators=[wtforms.validators.DataRequired()],
@@ -25,17 +30,26 @@ class ClientAdd(FlaskForm):
     )
     client_uri = wtforms.URLField(
         _("URI"),
-        validators=[wtforms.validators.DataRequired()],
+        validators=[
+            wtforms.validators.DataRequired(),
+            is_uri,
+        ],
         render_kw={"placeholder": "https://mydomain.tld"},
     )
     redirect_uris = wtforms.URLField(
         _("Redirect URIs"),
-        validators=[wtforms.validators.DataRequired()],
+        validators=[
+            wtforms.validators.DataRequired(),
+            is_uri,
+        ],
         render_kw={"placeholder": "https://mydomain.tld/callback"},
     )
     post_logout_redirect_uris = wtforms.URLField(
         _("Post logout redirect URIs"),
-        validators=[wtforms.validators.Optional()],
+        validators=[
+            wtforms.validators.Optional(),
+            is_uri,
+        ],
         render_kw={"placeholder": "https://mydomain.tld/you-have-been-disconnected"},
     )
     grant_types = wtforms.SelectMultipleField(
@@ -80,17 +94,26 @@ class ClientAdd(FlaskForm):
     )
     logo_uri = wtforms.URLField(
         _("Logo URI"),
-        validators=[wtforms.validators.Optional()],
+        validators=[
+            wtforms.validators.Optional(),
+            is_uri,
+        ],
         render_kw={"placeholder": "https://mydomain.tld/logo.png"},
     )
     tos_uri = wtforms.URLField(
         _("Terms of service URI"),
-        validators=[wtforms.validators.Optional()],
+        validators=[
+            wtforms.validators.Optional(),
+            is_uri,
+        ],
         render_kw={"placeholder": "https://mydomain.tld/tos.html"},
     )
     policy_uri = wtforms.URLField(
         _("Policy URI"),
-        validators=[wtforms.validators.Optional()],
+        validators=[
+            wtforms.validators.Optional(),
+            is_uri,
+        ],
         render_kw={"placeholder": "https://mydomain.tld/policy.html"},
     )
     software_id = wtforms.StringField(
@@ -110,7 +133,10 @@ class ClientAdd(FlaskForm):
     )
     jwks_uri = wtforms.URLField(
         _("JKW URI"),
-        validators=[wtforms.validators.Optional()],
+        validators=[
+            wtforms.validators.Optional(),
+            is_uri,
+        ],
         render_kw={"placeholder": ""},
     )
     preconsent = wtforms.BooleanField(
